import numpy as np
import numpy.ctypeslib as npct
import ctypes
from ctypes import c_int, c_bool, c_double, c_char

# Input types
ar_1d_double = npct.ndpointer(dtype=np.double, ndim=1, flags="CONTIGUOUS")
ar_1d_int = npct.ndpointer(dtype=np.int, ndim=1, flags="CONTIGUOUS")
ar_1d_bool = npct.ndpointer(dtype=np.bool, ndim=1, flags="CONTIGUOUS")
ar_2d_double = npct.ndpointer(dtype=np.double, ndim=2, flags="CONTIGUOUS")

# location
# get the correct location of the source files in folder master/DC/src
# load the library: dclib
dclib = ctypes.cdll.LoadLibrary(
    "c:\\Users\\hegek\\Documents\\NTNU\\5\\Master\\DC\\src\\dcflow.dll"
)

# restype og argtypes
dclib.secc_opt.restype = None
dclib.secc_opt.argtypes = (
    [c_int] * 4
    + [ar_1d_int]
    + [ar_1d_double] * 4
    + [ar_1d_int] * 3
    + [ar_1d_double]
    + [ar_1d_int]
    + [ar_1d_double]
    + [ar_1d_int] * 2
    + [ar_1d_double] * 2
    + [ar_1d_int]
    + [c_int] * 3
    + [ar_1d_int] * 2
    + [ar_1d_double] * 2
    + [ar_1d_int]
    + [ar_1d_double] * 3
    + [c_int] * 3
    + [ar_1d_double]
    + [ar_1d_int] * 2
    + [c_int] * 2
    + [ar_1d_int]
    + [c_int]
    + [ar_1d_int] * 2
    + [c_int]
    + [ar_1d_double]
    + [c_int]
    + [ar_1d_int] * 2
    + [ar_1d_double]
    + [c_double]
    + [ar_1d_int]
)

dclib.addbrc.restype = None
dclib.addbrc.argtypes = (
    [c_int] * 4
    + [ar_2d_double]
    + [ar_1d_double] * 3
    + [ar_1d_int] * 2
    + [ar_1d_double]
    + [c_int]
    + [ar_1d_int] * 3
    + [ar_1d_double] * 3
)

dclib.balcon.restype = None
dclib.balcon.argtypes = (
    [c_int] * 2
    + [ar_1d_double]
    + [c_int] * 1
    + [ar_2d_double]
    + [ar_1d_double] * 2
    + [ar_1d_int] * 2
    + [ar_1d_double] * 2
    + [c_int]
    + [ar_1d_int]
    + [ar_1d_double] * 2
)

dclib.remove.restype = None
dclib.remove.argtypes = (
    [c_int] * 4 + [ar_2d_double] + [ar_1d_double] * 2 + [ar_1d_int] * 2
)

dclib.upcon.restype = None
dclib.upcon.argtypes = (
    [c_int] * 3 + [ar_2d_double] + [ar_1d_double] * 3 + [ar_1d_int] + [ar_1d_double]
)

dclib.update.restype = None
dclib.update.argtypes = (
    [c_int] * 2 + [ar_1d_double] * 4 + [ar_1d_int] * 4 + [c_int] + [ar_1d_double] * 3
)

dclib.bulilist.restype = None
dclib.bulilist.argtypes = (
    [c_int]
    + [ar_1d_int]
    + [ar_1d_double] * 2
    + [ar_1d_int]
    + [c_int]
    + [ar_1d_int] * 2
    + [ar_1d_double] * 2
    + [ar_1d_int]
)

dclib.master.restype = None
dclib.master.argtypes = (
    [c_int] * 3
    + [ar_1d_int] * 3
    + [ar_1d_double] * 2
    + [ar_1d_int]
    + [ar_1d_double] * 6
    + [ar_1d_int] * 2
    + [ar_1d_double]
    + [ar_1d_int]
    + [ar_1d_double] * 2
    + [c_int]
    + [ar_1d_int] * 3
    + [ar_1d_double]
    + [ar_1d_int]
    + [c_double]
    + [ar_1d_int]
)


# function declaration
def secc_opt(
    lout,
    nbuses,
    ngens,
    nlines,
    igbus,
    pload,
    pgen,
    psell,
    pbuy,
    ifromb,
    itob,
    ickt,
    xinv,
    istat,
    ratea,
    ibusno,
    ibus,
    price,
    maxmw,
    segtyp,
    nseg,
    lmess,
    rmext,
    preffrst,
    prefnext,
    prefcost,
    prefmw,
    preftyp,
    teta0,
    basflow,
    cincr,
    icont,
    nicont,
    ibuild,
    a,
    irn,
    icn,
    nz,
    nsteps,
    ikeep,
    iflag,
    iw,
    iw1,
    maxfrt,
    w,
    la,
    wconct,
    wbus,
    akoeff,
    sbase,
    busnam,
):
    dclib.secc_opt(
        lout,
        nbuses,
        ngens,
        nlines,
        igbus,
        pload,
        pgen,
        psell,
        pbuy,
        ifromb,
        itob,
        ickt,
        xinv,
        istat,
        ratea,
        ibusno,
        ibus,
        price,
        maxmw,
        segtyp,
        nseg,
        lmess,
        rmext,
        preffrst,
        prefnext,
        prefcost,
        prefmw,
        preftyp,
        teta0,
        basflow,
        cincr,
        icont,
        nicont,
        ibuild,
        a,
        irn,
        icn,
        nz,
        nsteps,
        ikeep,
        iflag,
        iw,
        iw1,
        maxfrt,
        w,
        la,
        wconct,
        wbus,
        akoeff,
        sbase,
        busnam,
    )


def addbrc(
    il,
    nclin,
    nvar,
    mrowa,
    amat,
    bl,
    bu,
    sens,
    preffrst,
    prefnext,
    told,
    nbuses,
    ifromb,
    itob,
    igbus,
    xinv,
    pload,
    ratea,
):
    dclib.addbrc(
        il,
        nclin,
        nvar,
        mrowa,
        amat,
        bl,
        bu,
        sens,
        preffrst,
        prefnext,
        told,
        nbuses,
        ifromb,
        itob,
        igbus,
        xinv,
        pload,
        ratea,
    )


def balcon(
    nvar,
    nclin,
    cvec,
    mrowa,
    amat,
    bl,
    bu,
    preffrst,
    prefnext,
    prefcost,
    prefmw,
    nbuses,
    igbus,
    pload,
    akoeff,
):
    dclib.balcon(
        nvar,
        nclin,
        cvec,
        mrowa,
        amat,
        bl,
        bu,
        preffrst,
        prefnext,
        prefcost,
        prefmw,
        nbuses,
        igbus,
        pload,
        akoeff,
    )


def remove(ip, nvar, nclin, mrowa, amat, bl, bu, istate, iconp):
    dclib.remove(ip, nvar, nclin, mrowa, amat, bl, bu, istate, iconp)


def upcon(nvar, nclin, mrowa, amat, bl, bu, x, istate, told):
    dclib.upcon(nvar, nclin, mrowa, amat, bl, bu, x, istate, told)


def update(
    nbuses,
    ngens,
    cincr,
    pbuy,
    psell,
    pgen,
    igbus,
    preffrst,
    prefnext,
    preftyp,
    nvar,
    clamda,
    cvec,
    x,
):
    dclib.update(
        nbuses,
        ngens,
        cincr,
        pbuy,
        psell,
        pgen,
        igbus,
        preffrst,
        prefnext,
        preftyp,
        nvar,
        clamda,
        cvec,
        x,
    )


def bulilist(
    nbuses,
    ibus,
    price,
    maxmw,
    segtyp,
    nseg,
    preffrst,
    prefnext,
    prefcost,
    prefmw,
    preftyp,
):
    dclib.bulilist(
        nbuses,
        ibus,
        price,
        maxmw,
        segtyp,
        nseg,
        preffrst,
        prefnext,
        prefcost,
        prefmw,
        preftyp,
    )


def master(
    nbuses,
    ngens,
    nlines,
    igbus,
    preffrst,
    prefnext,
    prefcost,
    prefmw,
    preftyp,
    pload,
    cincr,
    pbuy,
    psell,
    pgen,
    teta0,
    ifromb,
    itob,
    xinv,
    istat,
    ratea,
    basflow,
    nlcold,
    ibusno,
    wconct,
    wbus,
    akoeff,
    ickt,
    sbase,
    busnam,
):
    dclib.master(
        nbuses,
        ngens,
        nlines,
        igbus,
        preffrst,
        prefnext,
        prefcost,
        prefmw,
        preftyp,
        pload,
        cincr,
        pbuy,
        psell,
        pgen,
        teta0,
        ifromb,
        itob,
        xinv,
        istat,
        ratea,
        basflow,
        nlcold,
        ibusno,
        wconct,
        wbus,
        akoeff,
        ickt,
        sbase,
        busnam,
    )

