__declspec(dllexport) void master(int nbuses, int ngens, int nlines, int *igbus, int *preffrst, int *prefnext, double *prefcost, double *prefmw, int *preftyp, double *pload, double *cincr, double *pbuy, double *psell, double *pgen, double *teta0, int *ifromb, int *itob, double *xinv, int *istat, double *ratea, double *basflow, int nlcold, int *ibusno, int *wconct, int *wbus, double *akoeff, int *ickt, double sbase, int *busnam)
{

#include "topflow.h"

    /* The routine solves the master problem in the security constrained dispatch problem */

    /* VARIABLE DECLARATIONS */
    /* Global variables from commonblocks: */

    int maxbus1, maclin2, licn, lirn, mrowa, mvar, mrsub, ndev0, liw, lin, lout, itmax, msglvl, nvar, nclin, nctotl, linobj, ifail, *istate, *iwork;

    double told, objlp, **amat, *bl, *bu, *cvec, *x, *clamda, *work;

    /* Declaration of local variables: */
    int ncnow, ib, ig, il, ifb, itb, *iconp, imaster;

    double *rside;

    // Number of main master problem iterations
    imaster = 0;

    // Set up the balance constraint, cost vector and variable limits
    if (nlcold == 0)
    {
        nclin = 0;
        balcon(nvar, nclin, cvec, mrowa, amat, bl, bu, preffrst, prefnext, prefcost, prefmw, nbuses, igbus, pload, akoeff);
    }

    // Skip the network analysis if NLINES is equal to 0. No network involved.
    if (nlines > 1)
    {

        printf("Solving Master Problem");

        do
        {

            // Find optimum with given constraints
            ncnow = nclin;
            ifail = 1;

            nctotl = nvar + nclin;

            imaster = imaster + 1;

            // Optimization algorithm
            // E04MBF(itmax, msglvl, nvar, nclin, nctotl, mrowa, amat, bl, bu, cvec, linobj, x, istate, objlp, clamda, iwork, liwork, work, lwork, ifail);

            printf("Iteration: %d. Power rescheduling cost: %f", imaster, objlp * sbase);

            /* Update upper and lower constraints if the problem is infeasible. Th option is only used when the constraints cannot be satisfied. The constraints are updated due to the close to optimal solution returned from the nag-routine. */
            if (ifail != 0)
            {
                upcon(nvar, nclin, mrowa, amat, bl, bu, x, istate, told);
                printf("Infeasible case - Constraint Relaxation applied in master problem");
            }

            // Update the generation vector
            update(nbuses, ngens, cincr, pbuy, psell, pgen, igbus, preffrst, prefnext, preftyp, nvar, clamda, cvec, x);

            // Remove nonbinding constraints in the optimization process
            remove(0, nvar, nclin, mrowa, amat, bl, bu, istate, iconp);

            // Find net injections on the nodes
            for (ib = 0; ib < nbuses; ib++)
            {
                ig = igbus[ib];
                if (ig > 0)
                {
                    teta0[ib] = pgen[ig] - pload[ib] - psell[ib] + pbuy[ib];
                }
                else
                {
                    teta0[ib] = -pload[ib] - psell[ib] + pbuy[ib];
                }
            }

            // Find voltage angles by a forward and a backward sunstitution
            // SOLVB(teta0, nbuses);

            // Identify overloads and add constraints
            for (il = 0; il < nbuses; il++)
            {
                ifb = ifromb[il];
                itb = itob[il];
                if (ratea[il] == 0.0)
                {
                    continue;
                }
                if (wbus[ifb] == 0 || wbus[itb] == 0)
                {
                    continue;
                }
                if (istat[il] == 1)
                {
                    basflow[il] = (teta0[ifb] - teta0[itb]) * xinv[il];
                    if ((ABS(basflow[il] - ratea[il])) > told)
                    {
                        addbrc(il, nclin, nvar, mrowa, amat, bl, bu, rside, preffrst, prefnext, told, nbuses, ifromb, itob, igbus, xinv, pload, ratea);

                        printf("Active transmission constraints after current iteration: \n From: %d %c to: %d %c, Ckt_Id: %d, Line flow: %f, Thermal rating: %f", ibusno[ifb], busnam[ifb], ibusno[itb], busnam[itb], ickt[il], basflow[il] * sbase, ratea[il] * sbase);
                    }
                }
                else
                {
                    basflow[il] = 0.0;
                }
            }

        } while (nclin > ncnow);

        for (il = 0; il < nlines; il++)
        {
            ifb = ifromb[il];
            itb = itob[il];
            if (ratea[il] == 0.0)
            {
                continue;
            }
            if (wbus[ifb] == 0 || wbus[itb] == 0)
            {
                continue;
            }
            if (istat[il] == 1)
            {
                if ((ABS(basflow[il] - ratea[il]) > -told))
                {
                    printf("Active transmission constraints after current iteration: \n From: %d %c to: %d %c, Ckt_Id: %d, Line flow: %f, Thermal rating: %f", ibusno[ifb], busnam[ifb], ibusno[itb], busnam[itb], ickt[il], basflow[il] * sbase, ratea[il] * sbase);
                }
            }
        }
    } //End if no network

    // Update the violation counter
    nlcold = nclin;

    // Dump incremental cost

} // End master
