C----------------------------------------------------------------C
C LIBRARY: STARTLIB   ! PROGRAM RESPONSIBLE:        !  Date:     C
C PROG.SYS: SECCON    !        OLAV BJARTE FOSSO    !  10.05.89  C
C----------------------------------------------------------------C

      SUBROUTINE BUILDB(BELEM,IROWP,JCOLP,IADR,NBUSES,
     &                  ISTAT,IFROMB,ITOB,ICKT,XINV,NLINES)
          
C----------------------------------------------------------------C
C DESCRIPTION:                                                   C
C     THE ROUTINE BUILDS THE SYSTEM DESCRIPTION B'.              C
C                                                                C
C----------------------------------------------------------------C

C----------------------------------------------------------------C
C  COMMENTS:                                                     C
C     ALL THE BUSES ARE PRESERVED SINCE THE SENSITIVITY          C
C     IS NEEDED FOR THE SWINGBUS TOO. NONSINGULARITY IS OBTAINED C
C     BY ADDING A LINE TO A FICTIVIOUS BUS.                      C
C                                                                C
C----------------------------------------------------------------C

C----------------------------------------------------------------C
C  INPUTDATA:                                                    C
C----------------------------------------------------------------C
C  NAME  !   TYPE  !  COMMENT                                    C
C----------------------------------------------------------------C
C NBUSES      I         NUMBER OF BUSES IN THE SYSTEM            C
C NLINES      I         NUMBER OF BRANCHES IN THE SYSTEM         C
C ISTAT()     I         VECTOR OF BRANCH STATUS                  C
C IFROMB()    I         VECTOR OF FROM BUS DEF. FOR BRANCHES     C
C ITOB()      I         VECTOR OF TO BUS DEF. FOR BRANCHES       C
C ICKT()      I         VECTOR OF BRANCH IDENTIFIERS             C
C XINV()      R         VECTOR OF BRANCH SUCEPTANCES             C
C                                                                C
C----------------------------------------------------------------C

C----------------------------------------------------------------C
C  OUTPUTDATA:                                                   C
C----------------------------------------------------------------C
C  NAME  !   TYPE  !  COMMENT                                    C
C----------------------------------------------------------------C
C BELEM()     R8        THE MATRIX ELEMENTS                      C
C IROWP()     I         VECTOR OF ROWPOSITIONS FOR BELEM()       C
C JCOLP()     I         VECTOR OF COLPOSITIONS FOR BELEM()       C
C IADR        I         NUMBER OF ELEMENTS IN BELEM()            C
C                                                                C
C----------------------------------------------------------------C

C----------------------------------------------------------------C
C  LOCAL VARIABLES:                                              C
C----------------------------------------------------------------C
C  NAME ! TYPE ! COMMENT                                         C
C----------------------------------------------------------------C
C IB      I       COUNTER FOR A LOOP FOR BUSES                   C
C ILINES  I       COUNTER FOR A LOOP FOR BRANCHES                C
C IFR     I       FROM-BUS FOR A SPECIFIC LINE                   C
C IT      I       TO-BUS FOR A SPECIFIC LINE                     C
C----------------------------------------------------------------C

C----------------------------------------------------------------C
C        DECLARATION OF MAINVARIABLES                            C
C----------------------------------------------------------------C

      REAL*8   BELEM(*)
      REAL     XINV(NLINES)
      INTEGER  ISTAT(NLINES)  ,
     &         IFROMB(NLINES)  , ITOB(NLINES)      , ICKT(NLINES)   ,
     &         IADR            , NBUSES            , NLINES

      INTEGER*2 IROWP(*)        , JCOLP(*)       

      

C----------------------------------------------------------------C

C----------------------------------------------------------------C
C        DECLARATION OF LOCAL VARIABLES                          C
C----------------------------------------------------------------C

      INTEGER  IB            , ILINES            , IFR              ,
     &         IT          

C----------------------------------------------------------------C

C
C  INITIALIZE  BELEM()
C
      DO 100 IB = 1,NBUSES+NLINES
        BELEM(IB) = 0.0
100   CONTINUE
C
C
      IADR = NBUSES
C
      DO 200 ILINES = 1,NLINES
         IF (ISTAT(ILINES) .NE. 1) GOTO 200
         IFR = IFROMB(ILINES)
         IT  = ITOB(ILINES)
         BELEM(IFR) = BELEM(IFR) + XINV(ILINES)
         IROWP(IFR) = IFR
         JCOLP(IFR) = IFR
C
         BELEM(IT) = BELEM(IT) + XINV(ILINES)
         IROWP(IT) = IT
         JCOLP(IT) = IT
C
C  ALREADY EXISTING ELEMENT?
C
         IF (ICKT(ILINES) .EQ. 1) GOTO 160
         DO 150 IB = IADR,NBUSES+1,-1
           IF ((IFR.EQ.IROWP(IB)) .AND. (IT.EQ.JCOLP(IB))) THEN
              BELEM(IB) = BELEM(IB) - XINV(ILINES)
              GOTO 200
           ENDIF
150      CONTINUE

C
C     ADD NEW ELEMENTS
C

160      CONTINUE
         IADR = IADR + 1
         BELEM(IADR) =  - XINV(ILINES)
         IROWP(IADR) = IFR
         JCOLP(IADR) = IT
C
200   CONTINUE

C
C  ADD DUMMY BRANCH FOR ENSURING FEASIBILITY
C
      BELEM(NBUSES) = BELEM(NBUSES) + 10

C
C   END
C

      END

