C----------------------------------------------------------------C
C LIBRARY: MASTERLIB  ! PROGRAM RESPONSIBLE:        !  Date:     C
C PROG.SYS: SECCON    !        OLAV BJARTE FOSSO    !  10.05.89  C
C----------------------------------------------------------------C

      SUBROUTINE MASTER(NBUSES,NGENS,NLINES,IGBUS,PREFFRST,PREFNEXT,
     &           PREFCOST,PREFMW,PREFTYP,PLOAD,CINCR,PBUY,PSELL,PGEN,
     &           TETA0,IFROMB,ITOB,XINV,ISTAT,RATEA,BASFLOW,NLCOLD,
     &           IBUSNO,LMESS,WCONCT,WBUS,AKOEFF,ICKT,SBASE,BUSNAM)

C----------------------------------------------------------------C
C DESCRIPTION:                                                   C
C     THE ROUTINE SOLVES THE MASTER PROBLEM IN THE SECURITY      C
C     CONSTRAINED DISPATCH PROBLEM.                              C
C                                                                C
C----------------------------------------------------------------C

C----------------------------------------------------------------C
C  COMMENTS:                                                     C
C                                                                C
C================================================================C
C  NAME  !   TYPE  !  COMMENT                                    C
C----------------------------------------------------------------C
C GLOBAL VARIABLES:                                              C
C                                                                C
C                                                                C
C COMMON BLOCKS                                                  C

      INCLUDE '../common/param.cmn'
      INCLUDE '../common/optim.cmn'
      INCLUDE '../common/scratch.cmn'


C----------------------------------------------------------------C
C  INPUTDATA:                                                    C
C----------------------------------------------------------------C
C  NAME  !   TYPE  !  COMMENT                                    C
C----------------------------------------------------------------C
C  NBUSES     I       NUMBER OF BUSES                            C
C  NGENS      I       NUMBER OF GENERATOR BUSES                  C
C  IGBUS()    I       POINTERS TO GENERATORS                     C
C  PREFFRST() I       POINTER TO FIRST SEGMENT IN PREF.FUNC.     C
C  PREFNEXT() I       POINTER TO NEXT SEGMENT                    C
C  PREFCOST() R       COST VECTOR FOR THE SEGMENTS               C
C  PREFMW()   R       MAXIMUM CAPACITY FOR SEGMENTS              C
C  PREFTYP    I       TYPE OF SEGMENT SELL = 1, BUY = 2, GEN = 3 C
C  PLOAD()    R       LOAD-VECTOR                                C
C  IFROMB()   R       FROM BUSES FOR BRANCHES                    C
C  ITOB()     R       TO BUSES FOR BRANCHES                      C
C  XINV()     R       INVERSE OF BRANCH REACTANCE                C
C  ISTAT()    I       STATUS FOR BRANCHES                        C
C  RATEA()    R       THERMAL RATING FOR BRANCH                  C
C                                                                C
C----------------------------------------------------------------C

C----------------------------------------------------------------C
C  OUTPUTDATA:                                                   C
C----------------------------------------------------------------C
C  NAME  !   TYPE  !  COMMENT                                    C
C----------------------------------------------------------------C
C  CINCR()    R       INCREMENTAL COST FOR BUSES MODELLED IN OPT C
C  PBUY()     R       BUYING OF POWER ON BUS                     C
C  PSELL()    R       SELLING                                    C
C  PGEN()     R       GENERATION                                 C
C  TETA0()    R       VOLTAGE ANGLES                             C
C  BASFLOW()  R       BASE CASE FLOW FOR ALL BRANCHES            C
C                                                                C
C----------------------------------------------------------------C

C----------------------------------------------------------------C
C  LOCAL VARIABLES:                                              C
C----------------------------------------------------------------C
C  NAME ! TYPE ! COMMENT                                         C
C----------------------------------------------------------------C
C TETA0   R8      THE RIGHT HAND SIDE WHEN SOLVING THE AX = B    C
C NCNOW   I       VARIABLE USED FOR NUMBER OF LINEAR CONSTRAINTS C
C IB      I       VARIABLE USED FOR INTERNAL BUSNUMBERS          C
C IG      I       VARIABLE USED FOR GENERATOR BUSES              C
C IFB     I       A SPECIFIC FROM-BUS FOR A BRANCH               C
C ITB     I       A SPECIFIC TO-BUS FOR A BRANCH                 C
C                                                                C
C----------------------------------------------------------------C

C----------------------------------------------------------------C
C        DECLARATION OF MAIN VARIABLES                           C
C----------------------------------------------------------------C

      INTEGER  NBUSES           , NGENS          , IGBUS(NBUSES)    ,
     &         PREFFRST(NBUSES) , PREFNEXT(*)    , PREFTYP(*)       ,
     &         IFROMB(NLINES)   , ITOB(NLINES)   , ISTAT(NLINES)    ,
     &         IBUSNO(NBUSES)   , WCONCT(*)      , WBUS(*)          ,
     &         ICKT(NLINES)
C
      REAL     PREFCOST(*)      , PREFMW(*)      , PLOAD(NBUSES)    ,
     &         CINCR(NBUSES)    , PBUY(NBUSES)   , PSELL(NBUSES)    ,
     &         PGEN(NGENS)      ,                  XINV(NLINES)     ,
     &         RATEA(NLINES)    , BASFLOW(NLINES), AKOEFF(*)        ,
     &         SBASE

      REAL*8   TETA0(NBUSES)

      CHARACTER*8   BUSNAM(NBUSES)


C----------------------------------------------------------------C

C----------------------------------------------------------------C
C        DECLARATION OF LOCAL VARIABLES                          C
C----------------------------------------------------------------C

      INTEGER  NCNOW            , IB                 , IG           ,
     &         IL               , IFB                , ITB          ,
     &         ICONP(MROWA)     , IMASTER

      REAL*8   RSIDE(MAXBUS1)

C----------------------------------------------------------------C


C Number of main master problem iterations
      IMASTER = 0

C
C set running conditions for the optimization algorithm
C
      ITMAX = 1000
      MSGLVL = 0
      LINOBJ = .TRUE.

C
C  UPDATE THE OUTPUT CHANNEL
C

      CALL X04ABF(1,LOUT)

C
C  add the equation    sum pgen  = sum pload  if neccessary
C  set up the cost vector 
C  set up limits for the variables
C

      IF (NLCOLD .EQ. 0) THEN
         NCLIN = 0
         CALL BALCON(NVAR,NCLIN,CVEC,MROWA,AMAT,BL,BU,
     &               PREFFRST,PREFNEXT,PREFCOST,PREFMW,
     &               NBUSES,IGBUS,PLOAD,AKOEFF)
      ENDIF


      WRITE(LMESS,118)
118   FORMAT(//,T20 ' *****  Solving Master Problem *****',/)


150   CONTINUE

C
C
C
C  find optimum with given constraints
C

      NCNOW = NCLIN
      IFAIL = 1

      NCTOTL = NVAR + NCLIN

      IMASTER = IMASTER + 1
C
      CALL E04MBF(ITMAX,MSGLVL,NVAR,NCLIN,NCTOTL,MROWA,
     &            AMAT,BL,BU,CVEC,LINOBJ,X,ISTATE,OBJLP,CLAMDA,
     &            IWORK,LIWORK,WORK,LWORK,IFAIL)
C
      WRITE(LMESS,123) IMASTER, OBJLP*SBASE
123   FORMAT(/,1X,I2,'. Iteration - Power Rescheduling Cost : ',
     &               F15.3,/)

      IF (IFAIL .EQ. 0) GOTO 250
C
C  update upper and lower constraints  if the problem is infeasible
C  The option is only used when the constraints can not be satiesfies
C  The constraints are updated due to the close to optimal solution
C  returned from the nag-routine
C
      CALL UPCON(NVAR,NCLIN,MROWA,AMAT,BL,BU,X,ISTATE,TOLD)
      WRITE(LMESS,210)
210   FORMAT(/,1X,'Infeasible case - Constraint Relaxation applied',
     &          ' in Master Problem',/)

250   CONTINUE

C
C   update the generation vector
C

      CALL UPDATE(NBUSES,NGENS,CINCR,PBUY,PSELL,PGEN,IGBUS,
     &            PREFFRST,PREFNEXT,PREFTYP,NVAR,CLAMDA,CVEC,X)
C
C Skip the network analysis if NLINES is equal to 0.
C No network involved.
C
      IF (NLINES .LE. 1) GOTO 999
C
C  remove nonbinding constraints in the optimization process
C

      CALL REMOVE(0,NVAR,NCLIN,MROWA,AMAT,BL,BU,ISTATE,ICONP)

C
C  find net injections on the nodes
C

      DO 300 IB = 1,NBUSES
         IG = IGBUS(IB) 
         IF (IG .GT. 0) THEN
           TETA0(IB) = PGEN(IG) - PLOAD(IB) - PSELL(IB) + PBUY(IB)
         ELSE
           TETA0(IB) = - PLOAD(IB) - PSELL(IB) + PBUY(IB)
         ENDIF
300   CONTINUE
C
C  find voltage angles by a forward and a backward substitutions
C

      CALL SOLVB(TETA0,NBUSES)

C
C identify overloads and add constraints
C

      DO 400 IL = 1,NLINES
        IFB = IFROMB(IL)
        ITB = ITOB(IL)
        IF (RATEA(IL) .EQ. 0.0) GOTO 400
        IF ((WBUS(IFB) .EQ. 0) .OR. (WBUS(ITB) .EQ. 0)) GOTO 400
        IF (ISTAT(IL) .EQ. 1) THEN
          BASFLOW(IL) = (TETA0(IFB) - TETA0(ITB))*XINV(IL)
          IF ((ABS(BASFLOW(IL))-RATEA(IL)).GT.TOLD(3)) THEN
            CALL ADDBRC(IL,NCLIN,NVAR,MROWA,AMAT,BL,BU,RSIDE,
     &                   PREFFRST,PREFNEXT,TOLD,
     &                   NBUSES,IFROMB,ITOB,IGBUS,XINV,PLOAD,RATEA)
           WRITE(LMESS,402) IBUSNO(IFB),BUSNAM(IFB),IBUSNO(ITB),
     &                    BUSNAM(ITB), ICKT(IL),
     &                     BASFLOW(IL)*SBASE, RATEA(IL)*SBASE
          ENDIF
        ELSE
           BASFLOW(IL) = 0.0
        ENDIF
400   CONTINUE
402   FORMAT(1X,'Account for  Violation on Line: ',/, 
     &     4X,I6,1X,A8,3X,I6,1X,A8,I3,
     &        ' Flow : ',F8.1,'  Rate : ',F8.1)

C
C  new violations encountered?
C

      IF (NCLIN .GT. NCNOW) GOTO 150


451   FORMAT(//,1X,'Active transmission ',
     &   'constraints after current iteration:',
     &   //,1x, 'From:',T20,'To:',T37,'Ckt_Id',
     &   T47,'Flow [MW]',T60,'Rate [MVA]')

      WRITE(LMESS,451)

      DO 450 IL = 1, NLINES
        IFB = IFROMB(IL)
        ITB = ITOB(IL)
        IF (RATEA(IL) .EQ. 0.0) GOTO 450
        IF ((WBUS(IFB) .EQ. 0) .OR. (WBUS(ITB) .EQ. 0)) GOTO 450
        IF (ISTAT(IL) .EQ. 1) THEN
          IF ((ABS(BASFLOW(IL))-RATEA(IL)).GT. -TOLD(3)) THEN
             WRITE(LMESS,452) IBUSNO(IFB),BUSNAM(IFB),IBUSNO(ITB),
     &                    BUSNAM(ITB), ICKT(IL),
     &                    BASFLOW(IL)*SBASE, RATEA(IL)*SBASE
          ENDIF
        ENDIF

450   CONTINUE

452   FORMAT(1X,I6,1X,A8,3X,I6,1X,A8,3X,I3,
     &        4X,F8.1,4X,F8.1)



999   CONTINUE
C
C  update the violation counter
C
      NLCOLD = NCLIN
C
C DUMP INCREMENTAL COST
C
      OPEN(29,FILE='inccost.res',STATUS='UNKNOWN')
      WRITE(29,*) ' OBJECTIVE FUNCTION : ', OBJLP
      DO 850 IB = 1, NVAR
         WRITE(29,851) IB, CVEC(IB), CLAMDA(IB), X(IB)
850   CONTINUE 
851   FORMAT(1X,I4,'  COST :',F8.3,'  LAMBDA : ', F9.4, '   X : ', F8.3)
      CLOSE(29)

C
C   END
C

      END

